﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;   ///< To use Toradex C Win32 DLLs.
using gpio_vcsharp_demo;

namespace vcsharp_matrixKeypad_demo
{
    public partial class Form1 : Form
    {
        private const int ROW1 = 98;
        private const int ROW2 = 133;
        private const int ROW3 = 103;
        private const int ROW4 = 101;
        private const int COL1 = 97;
        private const int COL2 = 85;
        private const int COL3 = 79;
        private const int COL4 = 45;
        private const int HIGH = 1;
        private const int LOW = 0;
        private const int GPIO = -1;
        private const int totalRows = 4;
        private const int totalColumns = 4;
        private const int keyStableHitTimeout = 20;     ///< Time-out in ms to ignore the bouncing of the keystroke.
                                                        ///
        private int[] colPin = new int[4]{COL1, COL2, COL3, COL4};
        private int[] rowPin = new int[4]{ROW1, ROW2, ROW3, ROW4};

        IntPtr handle = IntPtr.Zero;
        gpio.uIo ioPin = new gpio.uIo();
        const ushort ioColibriPin = 0x20;
        private bool anyBoxInGreenColor = false;
        private const TdxCommon.ParamStorageType storageType = TdxCommon.ParamStorageType.StoreVolatile;
        private TextBox[] textBoxArray = new TextBox[totalColumns * totalRows];

        public Form1()
        {
            InitializeComponent();
            handle = gpio.Gpio_Init(null);
            gpio.Gpio_Open(handle);
            ioPin.type = ioColibriPin;

            ioPin.number = COL1;
            /// Sets pin dedicated for column 1 as GPIO output logic LOW.
            gpio.Gpio_SetConfigString(handle, ioPin, null, "altfn=-1,dir=out", storageType);
            gpio.Gpio_SetLevel(handle, ioPin, gpio.tIoLevel.ioLow);

            ioPin.number = COL2;
            /// Sets pin dedicated for column 2 as GPIO output logic LOW.
            gpio.Gpio_SetConfigString(handle, ioPin, null, "altfn=-1,dir=out", storageType);
            gpio.Gpio_SetLevel(handle, ioPin, gpio.tIoLevel.ioLow);

            ioPin.number = COL3;
            /// Sets pin dedicated for column 3 as GPIO output logic LOW.
            gpio.Gpio_SetConfigString(handle, ioPin, null, "altfn=-1,dir=out", storageType);
            gpio.Gpio_SetLevel(handle, ioPin, gpio.tIoLevel.ioLow);

            ioPin.number = COL4;
            /// Sets pin dedicated for column 4 as GPIO output logic LOW.
            gpio.Gpio_SetConfigString(handle, ioPin, null, "altfn=-1,dir=out", storageType);
            gpio.Gpio_SetLevel(handle, ioPin, gpio.tIoLevel.ioLow);

            ioPin.number = ROW1;
            /// Sets pin dedicated for row 1 as GPIO input.
            gpio.Gpio_ConfigureAsGpio(handle, ioPin);
            gpio.Gpio_SetDir(handle, ioPin, gpio.tIoDirection.ioInput);

            ioPin.number = ROW2;
            /// Sets pin dedicated for row 2 as GPIO input.
            gpio.Gpio_ConfigureAsGpio(handle, ioPin);
            gpio.Gpio_SetDir(handle, ioPin, gpio.tIoDirection.ioInput);

            ioPin.number = ROW3;
            /// Sets pin dedicated for row 3 as GPIO input.
            gpio.Gpio_ConfigureAsGpio(handle, ioPin);
            gpio.Gpio_SetDir(handle, ioPin, gpio.tIoDirection.ioInput);

            ioPin.number = ROW4;
            /// Sets pin dedicated for row 4 as GPIO input.
            gpio.Gpio_ConfigureAsGpio(handle, ioPin);
            gpio.Gpio_SetDir(handle, ioPin, gpio.tIoDirection.ioInput);

            /// Collects all text-boxes in an array.
            textBoxArray[0] = textBox1;
            textBoxArray[1] = textBox2;
            textBoxArray[2] = textBox3;
            textBoxArray[3] = textBox4;
            textBoxArray[4] = textBox5;
            textBoxArray[5] = textBox6;
            textBoxArray[6] = textBox7;
            textBoxArray[7] = textBox8;
            textBoxArray[8] = textBox9;
            textBoxArray[9] = textBox10;
            textBoxArray[10] = textBox11;
            textBoxArray[11] = textBox12;
            textBoxArray[12] = textBox13;
            textBoxArray[13] = textBox14;
            textBoxArray[14] = textBox15;
            textBoxArray[15] = textBox16;
        }

        /// Sets a particular column as High.
        private void SelectCol(int column)
        {
            int columnCount = 0;

            for (columnCount = 0; columnCount < totalColumns; columnCount++)
            {
                ioPin.number = (ushort)colPin[columnCount];
                if (columnCount == column)
                {
                    /// Sets the column pin level as HIGH.
                    gpio.Gpio_SetLevel(handle, ioPin, gpio.tIoLevel.ioHigh);
                }
                else
                {
                    /// Sets the column pin level as LOW.
                    gpio.Gpio_SetLevel(handle, ioPin, gpio.tIoLevel.ioLow);
                }
            }
        }

        /// Scans for row that is High.
        private bool GetRow(ref int row)
        {
            int rowCount = 0;

            for (rowCount = 0; rowCount < totalRows; rowCount++)
            {
                ioPin.number = (ushort)rowPin[rowCount];
                /// Gets the level of the pin configured as row.
                if (gpio.Gpio_GetLevel(handle, ioPin) == gpio.tIoLevel.ioHigh)
                {
                    /// Time-out in ms to ignore the bouncing of the keystroke.
                    Thread.Sleep(keyStableHitTimeout);
                    /// Gets the level of the pin configured as row.
                    if (gpio.Gpio_GetLevel(handle, ioPin) == gpio.tIoLevel.ioHigh)
                    {
                        row = rowCount;
                        return true;
                    }
                }
            }
            return false;
        }

        /// Scans for key pressed on matrix keypad.
        private bool MatrixKeypadScan(ref int row, ref int column)
        {
            int columnCount = 0;
            bool retVal = false;
            row = 0;
            column = 0;
            while (columnCount < totalColumns)
            {
                SelectCol(columnCount);      ///< Sets column pin level High.
                retVal = GetRow(ref row);    ///< Scans for row that is High.
                if (retVal == true)
                {
                    column = columnCount;
                    break;
                }
                columnCount++;
                Thread.Sleep(5);
            }
            return retVal;
        }

        /// Shows result on UI.
        private void UpdateResult(int rowResult, int columnResult, bool keyPressed)
        {
            int count = 0;
            int resultToDisplay = 0;
            if (keyPressed == true)
            {
                /// Convert row and column number to match with text of text-boxes.
                resultToDisplay = ((rowResult + 1) * 10) + columnResult + 1;
                /// For example, if rowResult = 0 and columnResult = 0 => resultToDisplay = 11 i.e. First box selected.
            }
            for (count = 0; count < (totalColumns * totalRows); count++)
            {
                if (textBoxArray[count].Text == resultToDisplay.ToString())
                {
                    textBoxArray[count].BackColor = Color.YellowGreen;
                    anyBoxInGreenColor = true;
                }
                else
                {
                    textBoxArray[count].BackColor = Color.White;
                }
            }
        }

        private void scanButton_Click(object sender, EventArgs e)
        {
            if (scanButton.Text == "Start Scanning")
            {
                scanButton.Text = "Stop Scanning";
                updateData.Enabled = true;
            }
            else
            {
                updateData.Enabled = false;
                scanButton.Text = "Start Scanning";
                UpdateResult(0, 0, false);
            }
        }

        private void updateData_Tick(object sender, EventArgs e)
        {
            int rowOutput = 0;
            int columnOutput = 0;
            bool retVal = false;

            retVal = MatrixKeypadScan(ref rowOutput, ref columnOutput);
            if (retVal == true)
            {
                UpdateResult(rowOutput, columnOutput, true);
            }
            if (anyBoxInGreenColor == true && retVal == false)
            {
                anyBoxInGreenColor = false;
                UpdateResult(rowOutput, columnOutput, false);
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            updateData.Enabled = false;
            ///< De-initializes GPIO library and frees up system resources before closing the program.
            gpio.Gpio_Close(handle);
            gpio.Gpio_Deinit(handle);
        }
    }
}